Package: BP4RNAseq
Type: Package
Title: A babysitter's package for reproducible RNA-seq analysis
Version: 1.23.0
Authors@R: c(person(given = "Shanwen",
            family = "Sun",
            role = c("cre", "aut"),
            email = "sunshanwen@gmail.com"),
            person(given = "Lei",
                   family = "Xu",
                   role = "aut"),
            person(given = "Quan",
                   family = "Zou",
                   role = "aut"))
Description: An automated pipe for reproducible RNA-seq analysis with the minimal efforts from researchers. The package can process bulk RNA-seq data and single-cell RNA-seq data. You can only provide the taxa name and the accession id of RNA-seq data deposited in the National Center for Biotechnology Information (NCBI). After a cup of tea or longer, you will get formated gene expression data as gene count and transcript count based on both alignment-based and alignment-free workflows.
License: GPL-2
Encoding: UTF-8
Imports: dplyr, fastqcr, stringr, tidyr, stats, utils, magrittr,
        reticulate
Depends: R (>= 4.0.0)
SystemRequirements: UNIX, SRA Toolkit=2.10.3, Entrez Direct=13.3,
        FastQC=v0.11.9, Cutadapt=2.10, datasets, SAMtools=1.9,
        HISAT2=2.2.0, StringTie=2.1.1, Salmon=1.2.1
RoxygenNote: 7.1.1
biocViews: GeneExpressionWorkflow
Suggests: knitr, rmarkdown, testthat
VignetteBuilder: knitr
git_url: https://git.bioconductor.org/packages/BP4RNAseq
git_branch: devel
git_last_commit: c76cfd2
git_last_commit_date: 2026-04-28
Repository: Bioconductor 3.24
Date/Publication: 2026-05-05
NeedsCompilation: no
Packaged: 2026-05-05 17:05:04 UTC; biocbuild
Author: Shanwen Sun [cre, aut],
  Lei Xu [aut],
  Quan Zou [aut]
Maintainer: Shanwen Sun <sunshanwen@gmail.com>

# Generated by roxygen2: do not edit by hand

export(adapter_trim)
export(align_based_quan)
export(align_free_quan)
export(average)
export(check_dep)
export(down2quan)
export(down_Ref)
export(fastq2quan)
export(qc_test)
export(quality_trim)
export(scRNA_quan)
export(sra2fastq)
export(sra_download)
import(stringr)
importFrom(magrittr,'%>%')

<!-- README.md is generated from README.Rmd. Please edit that file -->

# BP4RNAseq

The assessment of gene expression is central to uncovering the functions
of the genome, understanding the regulation of development and
investigating the molecular mechanisms that underlie cancer and other
diseases. RNA-sequencing (RNA-seq) now is the routine to assess the
genome wide gene expression due to its high speed, accuracy and
reproducibility, and low cost. An enormous volume of RNA-seq data have
been accumulating and deposited in public data repositories, such as the
Gene Expression Omnibus (GEO) and the Sequence Read Archive (SRA).
Retrospectively analyzing these data or conducting a brand new RNA-seq
study is fundamentally important for researchers. However, processing
raw reads of RNA-seq data, no matter public or newly sequenced data,
involves a lot of specialized tools and technical configurations that
are often unfamiliar and time-consuming to learn for non-bioinformatics
researchers. For example, when working with public RNA-seq data,
researchers need to download the RNA-seq data, convert data to FASTQ
format, check the sequencing type (i.e., single-end or pair-end), do the
quality control (when needed, trim adapters and poor quality reads),
download the reference genome, transcript and annotation file, align
reads to the reference genome or transcript and quantify gene
expression, etc. These steps and the details that they involve are even
tedious for bioinformatic scientist. The goal of BP4RNAseq is to make
the RNA-seq analysis smooth and easy and to minimize efforts from
researchers. The package offers several benefits to researchers. First,
the package is a highly automated tool. It can take only two
nontechnical parameters and output six formatted gene expression
quantification at gene and transcript levels. Second, it improves the
accuracy and sensitivity of RNA-seq analyses by using an optimized
pipeline. Third, it offers individual tools to provide users full
control to fine tune precisely how individual steps are optimized. This
can allow users to inspect intermediate outputs and thus to further
improve the accuracy and sensitivity of RNA-seq analyses. Users can also
use the package as a toolbox to run the exact tools that suit their
needs. Last but not least, the package applies to both retrospective and
newly generated bulk RNA-seq data analyses and is also applicable for
single-cell RNA-seq analyses based on the Alevin algorithm \[1\].

## Operating System Requirements

BP4RNAseq runs in Windows (Subsystem for Linux), Linux and macOS.

## Dependencies

The BP4RNAseq requires the following utilities:

  - [SRA Toolkit
    \>= 2.10.3](https://trace.ncbi.nlm.nih.gov/Traces/sra/sra.cgi?view=toolkit_doc)
  - [Entrez Direct
    \>= 13.3](https://www.ncbi.nlm.nih.gov/books/NBK179288/)
  - [FastQC \>=
    v0.11.9](https://www.bioinformatics.babraham.ac.uk/projects/fastqc/)
  - [Cutadapt \>= 2.10](https://cutadapt.readthedocs.io/en/stable/)
  - [SAMtools \>= 1.9](http://www.htslib.org/)
  - [HISAT2 \>= 2.2.0](http://daehwankimlab.github.io/hisat2/)
  - [StringTie \>= 2.1.1](https://ccb.jhu.edu/software/stringtie/)
  - [Salmon \>= 1.2.1](https://combine-lab.github.io/salmon/)  
  - [R \>= 4.0.0](https://www.r-project.org/)

Users can install these dependencies manually.

Alternatively, we provide a bash script to aid users to install all the
dependencies based on [conda](https://docs.conda.io/en/latest/). The
script uses Wget, which is pre-installed on most Linux distributions
such as Windows Subsystem for Linux, to download conda. If wget is not
installed, users can easily install it with the following commands.

  - Installing Wget on Ubuntu and Debian or Windows Subsystem
    equivalents

<!-- end list -->

``` r
sudo apt-get update 
sudo apt-get install -y wget
```

  - Installing Wget on CentOS and Fedora or Windows Subsystem
    equivalents

<!-- end list -->

``` r
sudo yum install wget
```

  - Installing Wget on macOS with Homebrew

<!-- end list -->

``` r
brew install wget
```

With Wget installed, users can install all the dependencies with the
following commands:

``` r
wget https://raw.githubusercontent.com/sunshanwen/BP4RNAseq/master/install_depends.sh
chmod +x install_depends.sh
./install_depends.sh
./use_conda.sh
```

## Installation

<!-- You can install the released version of BP4RNAseq from [CRAN](https://CRAN.R-project.org) with: -->

<!-- ``` r -->

<!-- #install.packages("BP4RNAseq") # remove comments later -->

<!-- ``` -->

<!-- And the development version from [GitHub](https://github.com/) with: -->

<!-- ``` r -->

<!-- # install.packages("devtools") -->

<!-- devtools::install_github("sunshanwen/BP4RNAseq") -->

<!-- ``` -->

You can install BP4RNAseq from [GitHub](https://github.com/) with:

``` r
devtools::install_github("sunshanwen/BP4RNAseq")
```

## Usage

### Bulk RNA-seq analyses

The functions in BP4RNAseq are integrated into two main functions:
down2quan for public RNA-seq data, fastq2quan for newly generated
RNA-seq data.

down2quan requires no input data and can receive only two nontechnical
parameters. The parameter `accession` specifies the accession id of the
target public RNA-seq data in the Gene Expression Omnibus (GEO) or the
Sequence Read Archive (SRA). The accession id can be of a whole
‘BioProject’ or multiple ‘BioSample’. The parameter `taxa` offers the
scientific or common name of the organism investigated. A simple example

``` r
library(BP4RNAseq)
down2quan(accession=c("SRR11486115","SRR11486114"), taxa="Drosophila melanogaster")
```

will download the public RNA-seq data of two ‘BioSample’ with accession
id “SRR11486115” and “SRR11486114”, respectively, and the latest
reference genome, transcript and annotation data of Drosophila
melanogaster, do the quality control (filter out the poor-quality reads
and contaminations), reads alignments and gene expression quantification
based on both alignment-free and alignment-based workflows in the work
directory. During the quality control procedure, if the contamination of
the adapter exists, the program will automatically detect the adapter
sequence to trim. However, an option is given to the users to provide
the adapter sequence if they know it.

fastq2quan works with local RNA-seq data in fastq formats. It needs two
nontechnical parameters at a minimum, i.e., `taxa` as explained above
and `pair`which specifies the sequencing type with `single` for
single-end (SE) reads or `paired` for paired-end (PE) reads. Users
should place all the fastq files in the work directory. A simple example

``` r
library(BP4RNAseq)
fastq2quan(taxa="Drosophila melanogaster", pair = "single")
```

will download the latest reference genome, transcript and annotation
data of Drosophila melanogaster, and do the quality control, reads
alignments and gene expression quantification using the local RNA-seq
data based on both alignment-free and alignment-based workflows as the
program down2quan do.

Both programs support the parallel computing, which is specified by the
`threads` parameter.

Outputs from both functions are two gene count matrixes and two
transcript count matrixes based on the alignment-based workflow and the
alignment-free workflow, and corresponding average matrixes over two
workflows. These outputs can be directly processed with
[DESeq2](https://bioconductor.org/packages/release/bioc/html/DESeq2.html),
[edgeR](https://bioconductor.org/packages/release/bioc/html/edgeR.html)
or
[limma](https://bioconductor.org/packages/release/bioc/html/limma.html).
Researchers may use the averages for downstream analyses \[2\].
Alternatively, we recommend to decide the type of data to use based on
their consistencies with qPCR results if available or/and the results
from the downstream analyses.

### Single-cell RNA-seq analyses

down2quan and fastq2quan can also be extended to process single-cell
RNA-seq data by setting the `scRNA` parameter to be ‘TRUE’ and
specifying the protocols. Currently, dropseq, chromium and chromiumV3
are supported protocols. A simple example

``` r
library(BP4RNAseq)
down2quan(accession=c("SRR11402955","SRR11402974"), taxa="Homo sapiens", scRNA = TRUE, protocol = "dropseq")
```

will download the public single-cell RNA-seq data from two ‘BioSample’
with accession id “SRR11402955” and “SRR11402974”, respectively, and the
latest reference genome, transcript and annotation data of Homo sapiens,
do the quality control, reads alignments and gene expression
quantification based on the Alevin workflow.

Alternatively,

``` r
library(BP4RNAseq)
fastq2quan(taxa="Homo sapiens", scRNA = TRUE, protocol = "dropseq")
```

can preprocess local single-cell RNA-seq data in fastq formats. The data
are paired-end reads with one read containing cellular barcode and
unique molecule identifier (UMI) and the other read being the RNA
sequence.

The outputs of down2quan and fastq2quan are gene count matrix compressed
in binary format, and gene ids, barcode + UMI and tier categorization in
three separate files. These outputs can be further processed with
[tximport](https://bioconductor.org/packages/devel/bioc/vignettes/tximport/inst/doc/tximport.html)
and [Seurat](https://satijalab.org/seurat/).

### Using BP4RNAseq as a toolbox for RNA-seq analyses and customizing gene expression quantification to improve the sensitivity and accuracy of RNA-seq analyses.

BP4RNAseq offers individual tools to users. These tools can allow users
to run the exact tools that suit their needs. Specifically, these tools
are:

  - sra\_download(), which can download RNA-seq data from public
    repositories.
  - sra2fastq(), which converts SRA files into FASTQ format.
  - qc\_test(), which assesses the quality of reads.
  - adapter\_trim(), which trims adapter.
  - quality\_trim(), which trims poor quality reads.
  - down\_Ref(), which downloads the latest reference genome, transcript
    and annotation files.
  - align\_based\_quan(), which performs the alignment-based gene
    expression quantification, including reads alignment and assembly.
  - align\_free\_quan(), which performs the alignment-free gene
    expression quantification.
  - scRNA\_quan(), which performs the single-cell RNA-seq gene
    expression quantification.

Additionally, these individual tools provide users full control to fine
tune precisely how individual steps are optimized. This can allow
experienced users to further improve the accuracy and sensitivity of
RNA-seq analyses. For example, setting the optional parameter
`salmon_quan_add` of align\_free\_quan() as `salmon_quan_add = "--useEM
--gcBias"` will allow users to apply the standard EM algorithm to
optimize abundance estimates and in the mean time to correct for
fragment-level GC biases in the input data when performing the
alignment-free workflow. Details about the optional customizing setting
in each tool can be found in package help page.

## References

1.  Srivastava, A., et al. Alevin efficiently estimates accurate gene
    abundances from dscRNA-seq data. Genome Biol 2019;20:16.

2.  Lachmann, A., et al. Interoperable RNA-Seq analysis in the cloud.
    Biochim. Biophys. Acta-Gene Regul. Mech. 2020;1863(6):1-11.

#' Download RNA-seq samples from the NCBI.
#'
#' Download RNA-seq samples from the NCBI using SRA Toolkit and Entrez Direct based on the accession code.
#' @param accession the bioproject accession code for the RNA-seq samples deposited in the NCBI.
#' @param dir the working directory. Default is the current working directory.
#' @export sra_download
#' @return the status of the download (0 for success, 1 for error).
#' @examples
#' sra_download(accession = 'SRR11427582')
#'
### Re-code the function to make examples runnable

sra_download <- function(accession, dir = getwd()) {
    if (Sys.info()["sysname"] == "Windows") {
        print("Please use Windows subsystem for linux.")
        status <- 1
    } else {
        ### check if sratools is available or not.
        existence <- check_dep(dependancy = "vdb-config")
        if (existence == TRUE) {
            status <- 0
        } else {
            status <- 1
        }
        if (status == 0) {
            system2(command = "vdb-config", args = "--interactive | pidof")
            ## Get the default download directory
            dir.download <- NULL
            
            root.dir <- system2(command = "vdb-config", args = "| grep '<root>' | cut -d '>' -f 2 | cut -d '<' -f 1", stdout = TRUE)
            root.exist <- NULL
            for (i in seq_len(length(root.dir))) {
                if (root.dir[i] != ".") {
                  dir.download = root.dir[i]
                  root.exist <- TRUE
                }
            }
            if (length(dir.download) == 0) {
                dir.download <- system2(command = "vdb-config", args = "| grep '<HOME>' | cut -d '>' -f 2 | cut -d '<' -f 1", stdout = TRUE)
                root.exist <- FALSE
            }
            
            ### check if Entrez Direct tool is avaliable
            existence <- check_dep(dependancy = "esearch")
            #### start downloading
            if (existence == TRUE) {
                file_path <- NULL
                status <- NULL
                if (root.exist == TRUE) {
                  for (f in accession) {
                    #### use Entrez Direct tool to get all samples accession code, then use prefetch to download all the data
                    cmd2 = paste("$(esearch -db sra -query", f, "| efetch --format runinfo | grep", f, "| cut -d \",\" -f 1)")
                    status <- tryCatch(system2("prefetch", args = cmd2, stdout = FALSE, stderr = FALSE), error = function(err) {
                      1
                    }, warning = function(war) {
                      2
                    })
                    if (status == 0) {
                      file_path <- paste0(dir.download, "/sra")
                      file <- list.files(file_path, pattern = ".sra$", recursive = FALSE, full.names = TRUE)
                      files <- paste(file, collapse = " ")
                      cmd4 <- paste(files, "-t", dir)
                      system2(command = "mv", args = cmd4)
                    }
                  }
                  unlink(file_path)
                } else {
                  for (f in accession) {
                    #### use Entrez Direct tool to get all samples accession code, then use prefetch to download all the data
                    cmd2 = paste("$(esearch -db sra -query", f, "| efetch --format runinfo | grep", f, "| cut -d \",\" -f 1)")
                    status <- tryCatch(system2("prefetch", args = cmd2, stdout = FALSE, stderr = FALSE), error = function(err) {
                      1
                    }, warning = function(war) {
                      2
                    })
                    if (status == 0) {
                      file_path <- paste0(dir.download, "/", f)
                      file <- list.files(file_path, pattern = ".sra$", recursive = FALSE, full.names = TRUE)
                      files <- paste(file, collapse = " ")
                      cmd4 <- paste(files, "-t", dir)
                      system2(command = "mv", args = cmd4)
                    }
                  }
                  unlink(file_path)
                }
                if (status != 0) 
                  print("Poor internet connection. The download of RNAseq data failed. Please try it later.")
            } else print("Entrez Direct is not found. Please install it.")
        } else print("SRA Toolkit is not found. Please install it.")
    }
    return(status)
}

#' Trim poor quality samples using cutadapt.
#'
#' Trim poor quality samples using cutadapt based on the results from quality_test().
#' @param per_base samples with poor per base sequence quality.
#' @param pair 'single' for single-end (SE) or 'paired' for paired-end (PE) reads.
#' @param per_seq samples with poor per sequence quality scores.
#' @param threads the number of threads to be used. Default is 4.
#' @param scRNA logic, whether single-cell RNA-seq is quantified or not. Default is FALSE.
#' @param cutadapt_add additional parameters to customize cutadapt to trim poor quality reads. Default is NULL
#' @export quality_trim
#' @return None
#' @examples
#'
#' quality_trim(per_base = 'test_read_2.fastq', per_seq = 'test_read_2.fastq',
#'              pair = 'paired', threads = 4, scRNA = TRUE)
#'

quality_trim <- function(per_base, per_seq, pair, threads = 4, scRNA = FALSE, cutadapt_add = NULL) {
    if (scRNA == FALSE) {
        files <- unique(gsub("\\.fastq", "", unique(cbind(per_base, per_seq))))
        files <- unique(gsub("_1$", "", files))
        files <- unique(gsub("_2$", "", files))
        files <- unique(files)
        for (f in files) {
            if (pair == "paired") {
                file <- c(paste0(f, "_1_fastqc"), paste0(f, "_2_fastqc"))
                pattern <- paste0(f, "_1_fastqc.zip")
                qc_reports <- list.files(pattern = pattern, full.names = FALSE, recursive = FALSE)
                qc_collection <- fastqcr::qc_read_collection(qc_reports, sample_names = gsub(".zip", "", qc_reports), modules = "all", verbose = TRUE)
                Encoding <- unique(qc_collection$basic_statistics[qc_collection$basic_statistics$Measure == "Encoding" & qc_collection$basic_statistics$sample == 
                  file[1], 3])
                
                infile1 <- paste0(f, "_1.fastq")
                infile2 <- paste0(f, "_2.fastq")
                
                outfile1 <- paste0("Trimmed_", f, "_1.fastq")
                outfile2 <- paste0("Trimmed_", f, "_2.fastq")
                
                if (length(Encoding)) {
                  if (Encoding == "Sanger / Illumina 1.9") {
                    cmd = paste("-q 10", "-o", outfile1, "-p", outfile2, "-m 20", "-j ", threads, infile1, infile2)
                    system2(command = "cutadapt", args = cmd)
                  } else if (Encoding == "Illumina 1.3" | Encoding == "Illumina 1.5") {
                    cmd = paste("-q 10", "--quality-base=64", "-o", outfile1, "-p", outfile2, "-m 20", "-j ", threads, infile1, infile2)
                    system2(command = "cutadapt", args = cmd)
                  } else {
                    stop("Wrong Encoding. Exit.")
                  }
                }
            } else if (pair == "single") {
                file <- paste0(f, "_fastqc")
                pattern <- paste0(f, "_fastqc.zip")
                qc_reports <- list.files(pattern = pattern, full.names = FALSE, recursive = FALSE)
                qc_collection <- fastqcr::qc_read_collection(qc_reports, sample_names = gsub(".zip", "", qc_reports), modules = "all", verbose = TRUE)
                Encoding <- unique(qc_collection$basic_statistics[qc_collection$basic_statistics$Measure == "Encoding" & qc_collection$basic_statistics$sample == 
                  file, 3])
                
                infile <- paste0(f, ".fastq")
                outfile <- paste0("Trimmed_", f, ".fastq")
                
                if (length(Encoding)) {
                  if (Encoding == "Sanger / Illumina 1.9") {
                    cmd = paste("-q 10", "-o", outfile, "-m 20", "-j ", threads, infile)
                    system2(command = "cutadapt", args = cmd)
                  } else if (Encoding == "Illumina 1.3" | Encoding == "Illumina 1.5") {
                    cmd = paste("-q 10", "--quality-base=64", "-o", outfile, "-m 20", "-j ", threads, infile)
                    system2(command = "cutadapt", args = cmd)
                  } else {
                    stop("Wrong Encoding. Exit.")
                  }
                } else {
                  print("Could not get the Encoding.")
                }
            }
        }
        unlink(files)
    } else if (scRNA == TRUE) {
        files <- unique(gsub("\\.fastq", "", unique(c(per_base, per_seq))))
        for (f in files) {
            pattern <- paste0(f, "_fastqc.zip")
            qc_reports <- list.files(pattern = pattern, full.names = FALSE, recursive = FALSE)
            qc_collection <- fastqcr::qc_read_collection(qc_reports, sample_names = gsub(".zip", "", qc_reports), modules = "all", verbose = TRUE)
            file <- paste0(f, "_fastqc")
            Encoding <- unique(qc_collection$basic_statistics[qc_collection$basic_statistics$Measure == "Encoding" & qc_collection$basic_statistics$sample == 
                file, 3])
            infile <- paste0(f, ".fastq")
            outfile <- paste0("Trimmed_", f, ".fastq")
            
            if (length(Encoding)) {
                if (Encoding == "Sanger / Illumina 1.9") {
                  cmd = paste("-q 10", "-o", outfile, "-m 20", "-j ", threads, infile)
                  system2(command = "cutadapt", args = cmd)
                } else if (Encoding == "Illumina 1.3" | Encoding == "Illumina 1.5") {
                  cmd = paste("-q 10", "--quality-base=64", "-o", outfile, "-m 20", "-j ", threads, infile)
                  system2(command = "cutadapt", args = cmd)
                }
            } else {
                print("Could not get the Encoding.")
            }
        }
    }
}

#' Take averages with alignment-free and alignment-based quantification
#' @param based_gene gene count matrix assessed based on alignment-based workflow
#' @param free_gene gene count matrix assessed based on alignment-free workflow
#' @param based_transcript transcript count matrix assessed based on alignment-based workflow
#' @param free_transcript transcript count matrix assessed based on alignment-free workflow
#' @importFrom magrittr '%>%'
#' @export average
#' @return the average of gene expression quantification with alignment-free and alignment-based quantification workflow
#' @examples
#'
#' align_based_gene <- system.file(
#' 'extdata', 'test_gene_alignment_based_quantification.csv', 
#' package = 'BP4RNAseq'
#' )
#' align_free_gene <- system.file(
#' 'extdata', 'test_gene_alignment_free_quantification.csv', 
#' package = 'BP4RNAseq'
#' )
#' align_based_transcript <- system.file(
#' 'extdata', 'test_transcript_alignment_based_quantification.csv',
#' package = 'BP4RNAseq'
#' )
#' align_free_transcript <- system.file(
#' 'extdata', 'test_transcript_alignment_free_quantification.csv', 
#' package = 'BP4RNAseq'
#' )
# 
#' align_based_gene <- read.csv(align_based_gene, header = TRUE)
#' align_free_gene <- read.csv(align_free_gene, header = TRUE)
#' align_based_transcript <- read.csv(align_based_transcript, header = TRUE)
#' align_free_transcript <- read.csv(align_free_transcript, header = TRUE)

#' average(
#' align_based_gene, align_free_gene, 
#' align_based_transcript, align_free_transcript
#' )

average <- function(based_gene, free_gene, based_transcript, free_transcript) {
    ### gene count
    align_based_gene <- as.data.frame(based_gene)
    align_free_gene <- as.data.frame(free_gene)
    align_based_transcript <- as.data.frame(based_transcript)
    align_free_transcript <- as.data.frame(free_transcript)
    
    all_gene <- rbind(align_based_gene, align_free_gene)
    all_gene$sample <- as.factor(all_gene$sample)
    all_gene$gene_id <- as.factor(all_gene$gene_id)
    ave_gene <- all_gene %>% dplyr::group_by(sample, gene_id) %>% dplyr::summarise(count = mean(count, na.rm = TRUE))
    utils::write.csv(ave_gene, "average_gene_quantification.csv", row.names = FALSE)
    
    ### transcript count
    
    all_transcript <- rbind(align_based_transcript, align_free_transcript)
    all_transcript$sample <- as.factor(all_transcript$sample)
    all_transcript$transcript_id <- as.factor(all_transcript$transcript_id)
    ave_transcript <- all_transcript %>% dplyr::group_by(sample, transcript_id) %>% dplyr::summarise(count = mean(count, na.rm = TRUE))
    utils::write.csv(ave_transcript, "average_transcript_quantification.csv", row.names = FALSE)
}

tx2gene_scRNA <- function() {
    annotation <- list.files(pattern = "gff$", recursive = TRUE, full.names = TRUE)
    existence <- check_dep(dependancy = "egrep")
    if (existence == TRUE) {
        cmd1 <- paste("-v '^#|^$'", annotation, "| cut -f 9 | grep ID=rna | awk -F ';' 'BEGIN{OFS = \"=\";} {print $1, $2;}' | awk -F '=' 'BEGIN{OFS = \",\"} {print $NF, $2}' > raw_tx2gene.csv")
        system2(command = "egrep", args = cmd1)
        tx2gene <- utils::read.csv("raw_tx2gene.csv", header = FALSE)
        tx2gene[] <- lapply(tx2gene, as.character)
        index_to_be_changed <- which(tx2gene[, 1] %in% tx2gene[, 2])
        b <- length(index_to_be_changed)
        if (b > 0) {
            for (i in seq_len(b)) {
                tx2gene[index_to_be_changed[i], 1] <- tx2gene[tx2gene[, 2] == tx2gene[index_to_be_changed[i], 1], 1]
            }
        }
        tx2gene[, 1] <- gsub("gene-", "", tx2gene[, 1])
        tx2gene[, 2] <- gsub("rna-", "", tx2gene[, 2])
        tx2gene <- tx2gene[, c(2, 1)]
        utils::write.table(tx2gene, sep = "\t", col.names = FALSE, row.names = FALSE, "tx2gene.tsv", quote = FALSE)
        unlink("raw_tx2gene.csv")
    }
}


#' scRNA expression quantification with Salmon.
#' @param transcript the reference transcript
#' @param protocol the single-cell RNA sequencing protocol: dropseq, chromium, or chromiumV3.
#' @param threads the number of threads to be used. Default is 4.
#' @param salmon_index_add additional parameters to customize salmon index command. Default is NULL.
#' @param salmon_alevin_add additional parameters to customize salmon alevin command. Default is NULL.
#' @export scRNA_quan
#' @return None
#' @examples
#'
#' scRNA_quan(transcript = 'test_transcript_Homo_sapiens.fna', protocol = 'dropseq')
#'
scRNA_quan <- function(transcript, protocol, threads = 4, salmon_index_add = NULL, salmon_alevin_add = NULL) {
    existence <- check_dep(dependancy = "salmon")
    if (existence == TRUE) {
        tx2gene_scRNA()
        cmd3 <- paste("index -t", transcript, "-i salmon_index", "-p", threads, salmon_index_add)
        system2(command = "salmon", args = cmd3)
        read <- list.files(pattern = "^Trimmed.*1\\.fastq$", full.names = FALSE)
        if (length(read) == 0) {
            read <- list.files(pattern = ".*1\\.fastq$", full.names = FALSE)
        }
        for (f in read) {
            name <- gsub("_1.fastq", "", f)
            out <- paste0("scRNA_", name)
            read1 <- paste0(name, "_1.fastq")
            read2 <- paste0(name, "_2.fastq")
            read1 <- paste0(name, "_1.fastq")
            read2 <- paste0(name, "_2.fastq")
            read_seq <- read2
            barcode_seq <- read1
            cmd1 <- paste0("-n 1 ", read1, " | grep -o length=[0-9]* | cut -d '=' -f 2")
            leg_1 <- as.numeric(system2(command = "head", args = cmd1))
            cmd2 <- paste0("-n 1 ", read2, " | grep -o length=[0-9]* | cut -d '=' -f 2")
            leg_2 <- as.numeric(system2(command = "head", args = cmd2))
            if (leg_1 > leg_2) {
                read_seq <- read1
                barcode_seq <- read2
            }
            cmd4 <- paste("alevin -p", threads, "-i salmon_index -l ISR --", protocol, " -1 ", barcode_seq, " -2 ", read_seq, " --tgMap tx2gene.tsv -o ", 
                out, salmon_alevin_add)
            system2(command = "salmon", args = cmd4)
        }
        unlink("tx2gene.tsv")
        unlink("salmon_index", recursive = TRUE)
    } else print("Salmon is not found. Please install it.")
}

#' Produce the quantifications of gene expression based on the fastq files.
#'
#' Produce the quantifications of gene expression based on the fastq files with alignment-based and alignment-free workflows.
#'
#'@param threads the number of threads to be used. Default is 4.
#'@param dir the working directory. Default is the current working directory.
#'@param pair 'single' for single-end (SE) reads or 'paired' for paired-end (PE) reads.
#'@param taxa the scientific or common name of the organism.
#'@param novel_transcript logic, whether identifying novel transcripts is expected or not. Default is FALSE.
#'@param scRNA logic, whether single-cell RNA-seq is quantified or not. Default is FALSE.
#'@param protocol the single-cell RNA sequencing protocol: dropseq, chromium, or chromiumV3.
#'@export fastq2quan
#'@return None
#'@examples
#'
#'sra_download(accession = 'SRR11427582')
#'sra2fastq()
#'fastq2quan(pair = 'single', taxa = 'Drosophila melanogaster')


fastq2quan <- function(threads = 4, dir = getwd(), pair, taxa, novel_transcript = FALSE, scRNA = FALSE, protocol = NULL) {
    setwd(dir)
    status <- down_Ref(taxa)
    
    fastq.files <- list.files(pattern = "fastq$", full.names = FALSE)
    if (length(fastq.files)) {
        quality <- qc_test(threads, scRNA)
        
        if (length(quality$adapter$sample)) 
            cat("Adapter exists!\n")
        if (length(quality$base_quality$sample)) 
            cat("Per base sequence quality failed!\n")
        if (length(quality$sequence_score$sample)) 
            cat("Per sequence quality scores failed!\n")
        
        ### quality trimming
        if (length(quality$base_quality$sample) | length(quality$sequence_score$sample)) {
            print("Trim low quality bases.")
            quality_trim(quality$base_quality$sample, quality$sequence_score$sample, pair, scRNA)  ### consider if add directory parameter.
            if (length(quality$adapter$sample)) {
                print("Trim the adapter.")
                adapter_trim(quality$adapter$sample, pair, scRNA)
            }
        }
        
        if (length(quality$adapter$sample)) {
            print("Trim the adapter.")
            quality_trim(quality$adapter$sample, quality$adapter$sample, pair, scRNA)  ### consider if add directory parameter.
            adapter_trim(quality$adapter$sample, pair, scRNA)
        }
        files <- list.files(dir, pattern = "fastqc", full.names = FALSE)
        unlink(files)
        
        if (status == 0) {
            taxa_tmp <- gsub("\\s", "_", taxa)
            genome <- paste0(taxa_tmp, ".fna")
            transcript <- paste0("transcript_", taxa_tmp, ".fna")
            annotation <- paste0(taxa_tmp, ".gff")
            if (scRNA == FALSE) {
                align_based_quan(pair, taxa, genome, annotation, novel_transcript, threads)
                
                align_free_quan(pair, genome, transcript, annotation, threads)
                
                ### renaming results from trans_quan and align_free_quan first gene count
                align_based_gene <- utils::read.csv("gene_alignment_based_quantification.csv")
                align_based_gene <- align_based_gene[, c("sample", "gene_id", "count")]
                align_free_gene <- utils::read.csv("gene_alignment_free_quantification.csv")
                align_free_gene <- align_free_gene[, c("sample", "gene_id", "count")]
                utils::write.csv(align_based_gene, "gene_alignment_based_quantification.csv", row.names = FALSE)
                utils::write.csv(align_free_gene, "gene_alignment_free_quantification.csv", row.names = FALSE)
                ### then transcript count
                align_based_transcript <- utils::read.csv("transcript_alignment_based_quantification.csv")
                align_based_transcript <- align_based_transcript[, c("sample", "transcript_id", "count")]
                align_free_transcript <- utils::read.csv("transcript_alignment_free_quantification.csv")
                align_free_transcript <- align_free_transcript[, c("sample", "transcript_id", "count")]
                utils::write.csv(align_based_transcript, "transcript_alignment_based_quantification.csv", row.names = FALSE)
                utils::write.csv(align_free_transcript, "transcript_alignment_free_quantification.csv", row.names = FALSE)
                average(align_based_gene, align_free_gene, align_based_transcript, align_free_transcript)
            } else if (scRNA == TRUE) {
                scRNA_quan(transcript, protocol, threads)
            }
        }
    } else print("No fastq files are found in the work directory.")
}

#### Reads alignment with Hisat2
.index_build <- function(taxa, genome, annotation, threads, hisat2_build_add) {
    if (file.exists(genome) && file.exists(annotation)) {
        cmd1 <- paste("'{if ($3==\"exon\") {print $1\"\\t\"$4-1\"\\t\"$5-1}}'", annotation, "> exonsFile.table")
        system2(command = "awk", args = cmd1)
        cmd2 <- paste("'{if ($3==\"intron\") {print $1\"\\t\"$4-1\"\\t\"$5-1\"\\t\"$7}}'", annotation, "> ssFile.table")
        system2(command = "awk", args = cmd2)
        if (file.size("exonsFile.table") * file.size("ssFile.table")) {
            cmd3 <- paste("-f", genome, taxa, "--ss ssFile.table", "--exon exonsFile.table", "-p", threads, hisat2_build_add)
            system2(command = "hisat2-build", args = cmd3)
        } else {
            cmd3 <- paste("-f", genome, taxa, "-p", threads, hisat2_build_add)
            system2(command = "hisat2-build", args = cmd3)
        }
    } else print("The reference genome and annotation are missing. Please download them first.")
}

# Aligning the RNA-seq data to the reference genome with HISAT2.


.align_ge <- function(pair, taxa, genome, annotation, threads, hisat2_build_add, hisat2_add) {
    
    taxa <- gsub("\\s", "_", taxa)
    
    .index_build(taxa, genome, annotation, threads, hisat2_build_add)
    index <- list.files(pattern = "ht2$", recursive = TRUE, full.names = TRUE)
    if (length(index)) {
        if (pair == "paired") {
            read <- list.files(pattern = "^Trimmed.*1\\.fastq$", full.names = FALSE)
            if (length(read) == 0) {
                read <- list.files(pattern = ".*1\\.fastq$", full.names = FALSE)
            }
            if (length(read)) {
                for (f in read) {
                  name <- gsub("_1.fastq", "", f)
                  out_bam <- paste0(name, ".bam")
                  read1 <- paste0(name, "_1.fastq")
                  read2 <- paste0(name, "_2.fastq")
                  cmd4 <- paste("-p", threads, hisat2_add, "--dta -x", taxa, "-1", read1, "-2", read2, "| samtools view -bh - | samtools sort - >", out_bam)
                  system2(command = "hisat2", args = cmd4)
                }
            }
        } else if (pair == "single") {
            read <- list.files(pattern = "^Trimmed.*\\.fastq$", full.names = FALSE)
            if (length(read) == 0) {
                read <- list.files(pattern = ".*\\.fastq$", full.names = FALSE)
            }
            if (length(read)) {
                for (f in read) {
                  name <- gsub(".fastq", "", f)
                  out_bam <- paste0(name, ".bam")
                  cmd4 <- paste("-p", threads, hisat2_add, "--dta -x", taxa, "-U", f, "| samtools view -bh - | samtools sort - >", out_bam)
                  system2(command = "hisat2", args = cmd4)
                }
            }
        }
    }
}


# Transcript assembly with StringTie.

.trans_ass <- function(novel_transcript = FALSE, threads, stringtie_add) {
    aligned_bam <- list.files(pattern = "*\\.bam$")
    gff <- list.files(pattern = "gff$", recursive = TRUE, full.names = TRUE)
    if (length(aligned_bam) && length(gff)) {
        for (f in aligned_bam) {
            taxa <- gsub("\\.bam", "", f)
            output <- paste0("ballgown/", taxa, "/", taxa, ".gtf")
            if (novel_transcript == TRUE) {
                cmd1 <- paste(f, "-b ballgown -G", gff, "-o", output, "-p", threads, stringtie_add)
                system2(command = "stringtie", args = cmd1)
            } else {
                cmd1 <- paste(f, "-G", gff, "-e -b ballgown", "-o", output, "-p", threads, stringtie_add)
                system2(command = "stringtie", args = cmd1)
            }
        }
    }
}


#### Gene expression quantification based on the results from stringtie.

## extract the quantification data in gtf files produced by prepDE.py and save them to csv files.
.extract <- function() {
    files <- list.files(pattern = ".*\\.gtf$", recursive = TRUE, full.names = TRUE)
    outputs <- c()
    
    if (length(files)) {
        for (file in files) {
            output <- paste0("Tmp", gsub("\\.gtf$", "", gsub(".*/", "", file)), ".csv")
            outputs <- c(outputs, output)
            cmd = paste("-F '\\t' '$3 == \"transcript\" {print $9}'", file, "| awk -F ';' '{print $2, $5, $6}' | awk 'BEGIN{OFS = \",\"; print \"transcript_id\", \"FPKM\", \"TPM\"} {print $2, $4, $6}' >", 
                output)
            system2(command = "awk", args = cmd)
        }
    }
    return(outputs)
}

utils::globalVariables(c("transcript_id", "count", "gene_id"))

## comple all csv files into one file.
#' @importFrom magrittr '%>%'
#' @param files intermediate csv files

.compile <- function(files) {
    others <- data.frame(transcript_id = character(), FPKM = numeric(), TPM = numeric(), sample = character())
    for (f in files) {
        other <- utils::read.csv(f)
        other$sample <- gsub("\\.csv$", "", gsub("^Tmp", "", f))
        others <- rbind(others, other)
    }
    others <- stats::na.exclude(others)
    others <- others[!grepl("gene-", others$transcript_id), ]
    others$transcript_id <- sub("^.*?-", "", others$transcript_id)
    count <- utils::read.csv("transcript_count_matrix.csv")
    samples <- gsub("sorted_", "", names(count)[-1])
    names(count)[-1] <- samples
    count <- count[!grepl("gene-", count$transcript_id), ]
    count$transcript_id <- sub("^.*?-", "", count$transcript_id)
    tmp <- count %>% tidyr::gather(sample, count, -transcript_id)
    all <- merge(tmp, others, by = c("sample", "transcript_id"), all = TRUE)
    utils::write.csv(all, "transcript_alignment_based_quantification.csv")
}

# Expression quantification at gene and transcript levels.

#' @importFrom magrittr '%>%'
#'

.trans_quan <- function() {
    reticulate::source_python(system.file("prepDE_R.py", package = "BP4RNAseq"))
    outputs <- .extract()
    if (length(outputs)) {
        .compile(outputs)
        files <- list.files(pattern = "^Tmp.*csv$", recursive = TRUE, full.names = TRUE)
        unlink(files)
        files <- list.files(pattern = ".*bam$", recursive = TRUE, full.names = TRUE)
        unlink(files)
        files <- list.files(pattern = ".*ht2$", recursive = TRUE, full.names = TRUE)
        unlink(files)
        files <- list.files(pattern = ".*table$", recursive = TRUE, full.names = TRUE)
        unlink(files)
        gene_tmp <- utils::read.csv("gene_count_matrix.csv")
        gene_tmp <- gene_tmp %>% tidyr::gather(sample, count, -gene_id)
        gene_tmp$gene_id <- gsub("gene-", "", gene_tmp$gene_id)
        gene_tmp <- gene_tmp[, c("sample", "gene_id", "count")]
        utils::write.csv(gene_tmp, "gene_alignment_based_quantification.csv")
        unlink("gene_count_matrix.csv")
        unlink("transcript_count_matrix.csv")
        unlink("ballgown", recursive = TRUE)
    }
}

######## alignment-based workflow
#' alignment-based workflow
#'
#' @param pair 'single' for single-end (SE) or 'paired' for paired-end (PE) reads.
#' @param taxa the scientific or common name of the organism.
#' @param genome the reference genome.
#' @param annotation the annotation file.
#' @param novel_transcript logic, whether identifying novel transcripts is expected or not. Default is FALSE.
#' @param threads the number of threads to be used. Default is 4.
#' @param hisat2_build_add additional parameters to customize hisat2 build command. Default is NULL.
#' @param hisat2_add additional parameters to customize hisat2 command. Default is NULL.
#' @param stringtie_add additional parameters to customize stringtie command. Default is NULL.
#' @export align_based_quan
#' @return None
#' @examples
#' \dontrun{
#' test_Homo_sapiens.fna <- system.file('extdata', 'test_Homo_sapiens.fna', package = 'BP4RNAseq')
#' test_Homo_sapiens.gff <- system.file('extdata', 'test_Homo_sapiens.gff', package = 'BP4RNAseq')
#' align_based_quan(pair = 'paired', taxa = 'Homo sapiens', 
#' genome = test_Homo_sapiens.fna, 
#' annotation = test_Homo_sapiens.gff, novel_transcript = FALSE)
#'}
#'

align_based_quan <- function(pair, taxa, genome, annotation, novel_transcript = FALSE, threads = 4, hisat2_build_add = NULL, hisat2_add = NULL, stringtie_add = NULL) {
    existence <- check_dep(dependancy = "hisat2")
    if (existence == TRUE) {
        existence <- check_dep(dependancy = "stringtie")
        if (existence == TRUE) {
            .align_ge(pair, taxa, genome, annotation, threads, hisat2_build_add, hisat2_add)
            .trans_ass(novel_transcript, threads, stringtie_add)
            .trans_quan()
        }
    } else print("Hisat2 is not found. Please install it.")
}

## Get the adapter based on FASTQC report.

get_adapter <- function(sample, pair) {
    adapter <- NULL
    adap_3or5 <- NULL
    answer <- readline(prompt = "Do you know the adapter (y/n)? ")
    if (tolower(answer) == "y") {
        if (pair == "paired") {
            sequence1 <- readline(prompt = "Please enter the adapter sequence for read1: ")
            sequence2 <- readline(prompt = "Please enter the adapter sequence for read2: ")
            adapter <- c(sequence1, sequence2)
        } else if (pair == "single") {
            sequence <- readline(prompt = "Please enter the adapter sequence: ")
            adap_3or5 <- readline(prompt = "Please enter the adapter type (3 for 3' adapter types, 5 for 5' adapter types, 0 for Don't know): ")
            adapter <- sequence
        }
    } else {
        adapter_list = list(`Illumina Universal Adapter` = "AGATCGGAAGAG", `Illumina Small RNA 3' Adapter` = "TGGAATTCTCGG", `Nextera Transposase Sequence` = "CTGTCTCTTATA", 
            `SOLID Small RNA Adapter` = "CGCCTTGGCCGT", `Illumina Small RNA 5' Adapter` = "GATCGTCGGACT")
        pattern <- paste0(gsub("\\.fastq", "", sample), "_fastqc.zip")
        qc_reports <- list.files(pattern = pattern, full.names = FALSE)
        if (length(qc_reports)) {
            qc_collection <- fastqcr::qc_read_collection(qc_reports, sample_names = gsub(".zip", "", qc_reports), modules = "all", verbose = TRUE)
            if (length(sample)) {
                dat <- qc_collection$adapter_content
                tt <- apply(dat[, !(names(dat) %in% c("sample", "Position"))] > 10, 2, sum) > 0
                adapter_name <- names(tt)[tt]
                adapter <- adapter_list[[adapter_name]]
                if (adapter_name == "Illumina Small RNA 5' Adapter") {
                  adap_3or5 <- "5"
                } else {
                  adap_3or5 <- "3"
                }
            }
        }
    }
    return(list(adapter = adapter, adap_3or5 = adap_3or5))
}

#' Trim samples with adapter using cutadapt.
#'
#' Trim samples with adapter using cutadapt based on the results from quality_test().
#' @param sample samples with adapter.
#' @param pair 'single' for single-end (SE) or 'paired' for paired-end (PE) reads.
#' @param threads the number of threads to be used. Default is 4.
#' @param scRNA logic, whether single-cell RNA-seq is quantified or not. Default is FALSE.
#' @param cutadapt_add additional parameters to customize cutadapt to trim adapter. Default is NULL.
#' @export adapter_trim
#' @return None
#' @examples
#'
#' adapter_trim(sample = 'test_read_2.fastq', pair = 'paired', scRNA = TRUE)
#'
adapter_trim <- function(sample, pair, threads = 4, scRNA = FALSE, cutadapt_add = NULL) {
    existence <- check_dep(dependancy = "cutadapt")
    if (existence == TRUE) {
        if (scRNA == FALSE) {
            adapter_got = get_adapter(sample, pair)
            samples <- unique(gsub("_2$", "", gsub("_1$", "", gsub("\\.fastq", "", sample))))
            if (pair == "paired") {
                if (length(adapter_got$adapter)) {
                  for (i in samples) {
                    outfile1 <- paste0("Adapter_trimmed_", i, "_1.fastq")
                    outfile2 <- paste0("Adapter_trimmed_", i, "_2.fastq")
                    infile1 <- paste0("Trimmed_", i, "_1.fastq")
                    infile2 <- paste0("Trimmed_", i, "_2.fastq")
                    if (length(adapter_got$adapter) == 1) {
                      cmd = paste("-a", adapter_got$adapter, "-A", adapter_got$adapter, "-o", outfile1, "-p", outfile2, "-j ", threads, infile1, infile2, 
                        cutadapt_add)
                      system2(command = "cutadapt", args = cmd)
                      unlink(c(infile1, infile2))
                    } else if (length(adapter_got$adapter) == 2) {
                      arguments = paste("-a", adapter_got$adapter[1], "-A", adapter_got$adapter[2], "-o", outfile1, "-p", outfile2, "-j ", threads, infile1, 
                        infile2, cutadapt_add)
                      system2(command = "cutadapt", args = arguments)
                      unlink(c(infile1, infile2))
                    } else stop("Wrong adapter. Exit.")
                  }
                }
            } else if (pair == "single") {
                infile <- paste0("Trimmed_", i, ".fastq")
                outfile <- paste0("Adapter_trimmed_", i, ".fastq")
                if (length(adapter_got$adap_3or5)) {
                  if (adapter_got$adap_3or5 == "3") {
                    cmd = paste("-a", adapter_got$adapter, "-o", outfile, "-j ", threads, infile, cutadapt_add)
                    system2(command = "cutadapt", args = cmd)
                    unlink(infile)
                  } else if (adapter_got$adap_3or5 == "5") {
                    cmd = paste("-g", adapter_got$adapter, "-o", outfile, "-j ", threads, infile, cutadapt_add)
                    system2(command = "cutadapt", args = cmd)
                    unlink(infile)
                  } else if (adapter_got$adap_3or5 == "0") {
                    cmd = paste("-b", adapter_got$adapter, "-o", outfile, "-j ", threads, infile, cutadapt_add)
                    system2(command = "cutadapt", args = cmd)
                    unlink(infile)
                  } else stop("Wrong selection. Exit.")
                }
            }
        } else if (scRNA == TRUE) {
            samples <- unique(gsub("\\.fastq", "", sample))
            adapter_got = get_adapter(sample, pair = "single")
            for (i in samples) {
                infile <- paste0("Trimmed_", i, ".fastq")
                outfile <- paste0("Adapter_trimmed_", i, ".fastq")
                if (length(adapter_got$adap_3or5)) {
                  if (adapter_got$adap_3or5 == "3") {
                    cmd = paste("-a", adapter_got$adapter, "-o", outfile, "-j ", threads, infile, cutadapt_add)
                    system2(command = "cutadapt", args = cmd)
                    unlink(infile)
                  } else if (adapter_got$adap_3or5 == "5") {
                    cmd = paste("-g", adapter_got$adapter, "-o", outfile, "-j ", threads, infile, cutadapt_add)
                    system2(command = "cutadapt", args = cmd)
                    unlink(infile)
                  } else if (adapter_got$adap_3or5 == "0") {
                    cmd = paste("-b", adapter_got$adapter, "-o", outfile, "-j ", threads, infile, cutadapt_add)
                    system2(command = "cutadapt", args = cmd)
                    unlink(infile)
                  } else stop("Wrong selection. Exit.")
                }
            }
        }
    } else print("Cutadapt is not found. Please install it.")
}

#' Check the existence of dependancies.
#'
#' Trim samples with adapter using cutadapt based on the results from quality_test().
#' @param dependancy the dependancy to check.
#' @export check_dep
#' @return TRUE if the dependancy exists, otherwise FALSE. 
#' @examples
#'
#' check_dep(dependancy = 'cutadapt')
#'

check_dep <- function(dependancy) {
    status <- tryCatch(system2(command = "which", args = dependancy, stdout = FALSE, stderr = FALSE), error = function(err) {
        1
    }, warning = function(war) {
        2
    })
    if (status == 0) 
        return(TRUE) else return(FALSE)
}

convert_data <- function() {
    files <- list.files(pattern = "quant.sf", recursive = TRUE, full.names = TRUE)
    if (length(files)) {
        others <- data.frame(transcript_id = character(), length = numeric(), TPM = numeric(), count = numeric(), sample = character())
        for (f in files) {
            other <- utils::read.table(f, header = TRUE, sep = "\t")
            other$sample <- gsub("^\\./", "", gsub("_trans.*", "", f))
            others <- rbind(others, other[, -2])
        }
        names(others) <- c("transcript_id", "length", "TPM", "count", "sample")
        others <- others[, c("sample", "transcript_id", "count", "TPM", "length")]
        utils::write.csv(others, "transcript_alignment_free_quantification.csv", row.names = FALSE)
    }
}

tx2gene <- function() {
    annotation <- list.files(pattern = "gff$", recursive = TRUE, full.names = TRUE)
    if (length(annotation)) {
        cmd1 <- paste("-v '^#|^$'", annotation, "| cut -f 9 | grep ID=rna | awk -F ';' 'BEGIN{OFS = \"=\";} {print $1, $2;}' | awk -F '=' 'BEGIN{OFS = \",\"} {print $NF, $2}' > raw_tx2gene.csv")
        system2(command = "egrep", args = cmd1)
        ### add support for Ensembl annotation file
        if (file.info("raw_tx2gene.csv")$size == 0) {
            cmd1 <- paste("-v '^#|^$'", annotation, "| cut -f 9 | grep ID=transcript | awk -F ';' 'BEGIN{OFS = \":\";} {print $1, $2;}' | awk -F ':' 'BEGIN{OFS = \",\"} {print $NF, $2}' > raw_tx2gene.csv")
            system2(command = "egrep", args = cmd1)
        }
        tx2gene <- utils::read.csv("raw_tx2gene.csv", header = FALSE)
        tx2gene[] <- lapply(tx2gene, as.character)
        index_to_be_changed <- which(tx2gene[, 1] %in% tx2gene[, 2])
        b <- length(index_to_be_changed)
        if (b > 0) {
            for (i in seq_len(b)) {
                tx2gene[index_to_be_changed[i], 1] <- tx2gene[tx2gene[, 2] == tx2gene[index_to_be_changed[i], 1], 1]
            }
        }
        tx2gene[, 1] <- gsub("gene-", "", tx2gene[, 1])
        tx2gene[, 2] <- gsub("rna-", "", tx2gene[, 2])
        tx2gene <- tx2gene[, c(2, 1)]
        colnames(tx2gene) <- c("transcript_id", "gene_id")
        utils::write.csv(tx2gene, row.names = FALSE, "tx2gene.csv")
    }
}

utils::globalVariables("TPM")

gene_quan <- function() {
    if (file.exists("tx2gene.csv") && file.exists("transcript_alignment_free_quantification.csv")) {
        tx2gene_file <- utils::read.csv("tx2gene.csv")
        transcript <- utils::read.csv("transcript_alignment_free_quantification.csv")
        all <- merge(tx2gene_file, transcript, by = "transcript_id", all = TRUE)
        all <- stats::na.omit(all)
        tmp1 <- all %>% dplyr::group_by(sample, gene_id) %>% dplyr::summarise(count = sum(count))
        gene_quantification <- tmp1[, c("sample", "gene_id", "count")]
        utils::write.csv(gene_quantification, "gene_alignment_free_quantification.csv", row.names = FALSE)
    }
}

#' Alignment-free expression quantification with Salmon at gene and transcript levels.
#'
#' Alignment-free expression quantification with Salmon at gene and transcript levels.
#' @param pair 'single' for single-end (SE) or 'paired' for paired-end (PE) reads.
#' @param genome the reference genome.
#' @param transcript the reference transcript
#' @param annotation the annotation file.
#' @param threads the number of threads to be used. Default is 4.
#' @param salmon_index_add additional parameters to customize salmon index command. Default is NULL.
#' @param salmon_quan_add additional parameters to customize salmon quan command. Default is NULL.
#' @export align_free_quan
#' @return None
#' @examples
#'
#' align_free_quan(
#' pair = 'paired', 'test_Homo_sapiens.fna', 
#' 'test_transcript_Homo_sapiens.fna','test_Homo_sapiens.gff'
#' )
#'
#'
#'
align_free_quan <- function(pair, genome, transcript, annotation, threads = 4, salmon_index_add = NULL, salmon_quan_add = NULL) {
    existence <- check_dep(dependancy = "salmon")
    if (existence == TRUE) {
        if (file.exists(genome) && file.exists(transcript) && file.exists(annotation)) {
            cmd3 <- paste("index -t", transcript, "-i salmon_index", "-p", threads, salmon_index_add)
            system2(command = "salmon", args = cmd3)
            if (pair == "paired") {
                read <- list.files(pattern = "^Trimmed.*1\\.fastq$", full.names = FALSE)
                if (length(read) == 0) {
                  read <- list.files(pattern = ".*1\\.fastq$", full.names = FALSE)
                }
                if (length(read)) {
                  for (f in read) {
                    name <- gsub("_1.fastq", "", f)
                    out <- paste0(name, "_transcripts_quant")
                    read1 <- paste0(name, "_1.fastq")
                    read2 <- paste0(name, "_2.fastq")
                    cmd4 <- paste("quant -p", threads, "-i salmon_index -l A", "-1", read1, "-2", read2, "--validateMappings -o", out, salmon_quan_add)
                    system2(command = "salmon", args = cmd4)
                  }
                } else print("No fastq files are found in the work directory.")
            } else if (pair == "single") {
                read <- list.files(pattern = "^Trimmed.*\\.fastq$", full.names = FALSE)
                if (length(read) == 0) {
                  read <- list.files(pattern = ".*\\.fastq$", full.names = FALSE)
                }
                if (length(read)) {
                  for (f in read) {
                    name <- gsub(".fastq", "", f)
                    out <- paste0(name, "_transcripts_quant")
                    cmd4 <- paste("quant -p", threads, "-i salmon_index -l A -r", f, "--validateMappings -o", out, salmon_quan_add)
                    system2(command = "salmon", args = cmd4)
                  }
                }
            } else stop("Paired-end and single-end mix. Please check the data source!")
            convert_data()  #### rna quantification to use for quantifying genes.
            tx2gene()
            gene_quan()
            unlink("raw_tx2gene.csv")
            unlink("tx2gene.csv")
            folders <- dir(pattern = "transcripts_quant$")
            unlink(folders, recursive = TRUE)
            unlink("salmon_index", recursive = TRUE)
        } else print("No reference genome and annotation files are found. Please download them first")
    } else print("Salmon is not found. Please install it.")
}

#' @order 2
check_paired <- function(files) {
    temp <- data.frame(samples = NA, paired_or_single = NA)
    for (f in files) {
        cmd = paste("-X", "1", "-Z", "--split-spot", f, "| wc -l")
        dd <- system2(command = "fastq-dump", args = cmd)  # invoke command\
        if (dd == "8") {
            temp <- rbind(temp, c(f, "paired"))
        } else if (dd == "4") {
            temp <- rbind(temp, c(f, "single"))
        } else temp <- rbind(temp, c(f, "single"))
    }
    temp <- temp[-1, ]
    temp$paired_or_single <- as.factor(temp$paired_or_single)
    if (levels(temp$paired_or_single) == "paired") {
        return("paired")
    } else if (levels(temp$paired_or_single) == "single") {
        return("single")
    } else stop("Paired-end and single-end mix. Please check the data source!")
}

#' Convert SRA format to fastq format and return the sequencing type, i.e., single-end (SE) or paired-end (PE) reads.
#'
#' Convert all SRA files to fastq files and return the sequencing type, i.e., single-end (SE) or paired-end (PE) reads.
#' @param threads the number of threads to be used. Default is 4.
#' @import stringr
#' @export sra2fastq
#' @return A string indicates single-end (SE) or paired-end (PE) reads.
#' @order 1
#' @examples
#'
#' sra_download(accession = 'SRR11427582')
#' sra2fastq()
#'
sra2fastq <- function(threads = 4) {
    existence <- check_dep(dependancy = "fasterq-dump")
    if (existence == TRUE) {
        sra.dir = getwd()  ### get the directory to search SRA files
        fq.dir = getwd()
        files <- list.files(sra.dir, pattern = ".sra$", recursive = FALSE, full.names = FALSE)
        if (length(files)) {
            pair <- check_paired(files)
            for (f in files) {
                cmd = paste(f, "-O", fq.dir, "-e", threads)
                system2(command = "fasterq-dump", args = cmd)  # invoke command
                unlink(f)
            }
            
            fastq_files <- list.files(sra.dir, pattern = "fastq$", recursive = FALSE, full.names = FALSE)
            file.rename(fastq_files, gsub(".sra", "", fastq_files))
            unlink("sra", recursive = TRUE)
            return(pair)
        } else {
            print("No SRA files. Please download them.")
            return(NULL)
        }
    } else {
        print("fasterq-dump is not available. Please install the latest SRA Toolkit.")
        return(NULL)
    }
}

#' Download the RNA-seq samples from the NCBI and produce the quantifications of gene expression.
#'
#'Download the RNA-seq samples from the NCBI and produce the quantifications of gene expression with alignment-based and alignment-free workflows.
#'
#'@param accession the bioproject accession code for the RNA-seq samples deposited in the NCBI.
#'@param dir the working directory. Default is the current working directory.
#'@param taxa the scientific or common name of the organism.
#'@param novel_transcript logic, whether identifying novel transcripts is expected or not. Default is FALSE.
#'@param threads the number of threads to be used. Default is 4.
#'@param scRNA logic, whether single-cell RNA-seq is quantified or not. Default is FALSE.
#'@param protocol the single-cell RNA sequencing protocol: dropseq, chromium, or chromiumV3.
#'@export down2quan
#'@return None
#'@examples
#'
#' down2quan(
#' accession = 'SRR11427582', dir = getwd(), 
#' taxa = 'Drosophila melanogaster', novel_transcript = FALSE, scRNA = FALSE
#' )
#'

down2quan <- function(accession, dir = getwd(), taxa, novel_transcript = FALSE, threads = 4, scRNA = FALSE, protocol) {
    setwd(dir)
    status <- sra_download(accession, dir)
    if (status == 0) {
        pair <- sra2fastq(threads)
        if (length(pair)) {
            quality <- qc_test(threads, scRNA)
            
            if (length(quality$adapter$sample)) 
                cat("Adapter exists!\n")
            if (length(quality$base_quality$sample)) 
                cat("Per base sequence quality failed!\n")
            if (length(quality$sequence_score$sample)) 
                cat("Per sequence quality scores failed!\n")
            
            ### quality trimming
            if (length(quality$base_quality$sample) | length(quality$sequence_score$sample)) {
                print("Trim low quality bases.")
                quality_trim(quality$base_quality$sample, quality$sequence_score$sample, pair, scRNA)  ### consider if add directory parameter.
                if (length(quality$adapter$sample)) {
                  print("Trim the adapter.")
                  adapter_trim(quality$adapter$sample, pair, scRNA)
                }
            }
            
            if (length(quality$adapter$sample)) {
                print("Trim the adapter.")
                quality_trim(quality$adapter$sample, quality$adapter$sample, pair, scRNA)  ### consider if add directory parameter.
                adapter_trim(quality$adapter$sample, pair, scRNA)
            }
            files <- list.files(dir, pattern = "fastqc", full.names = FALSE)
            unlink(files)
            
            status <- down_Ref(taxa)
            if (status == 0) {
                taxa_tmp <- gsub("\\s", "_", taxa)
                genome <- paste0(taxa_tmp, ".fna")
                transcript <- paste0("transcript_", taxa_tmp, ".fna")
                annotation <- paste0(taxa_tmp, ".gff")
                if (scRNA == FALSE) {
                  align_based_quan(pair, taxa, genome, annotation, novel_transcript, threads)
                  
                  align_free_quan(pair, genome, transcript, annotation, threads)
                  
                  ### renaming results from trans_quan and align_free_quan first gene count
                  align_based_gene <- utils::read.csv("gene_alignment_based_quantification.csv")
                  align_based_gene <- align_based_gene[, c("sample", "gene_id", "count")]
                  align_free_gene <- utils::read.csv("gene_alignment_free_quantification.csv")
                  align_free_gene <- align_free_gene[, c("sample", "gene_id", "count")]
                  utils::write.csv(align_based_gene, "gene_alignment_based_quantification.csv", row.names = FALSE)
                  utils::write.csv(align_free_gene, "gene_alignment_free_quantification.csv", row.names = FALSE)
                  
                  ### then transcript count
                  align_based_transcript <- utils::read.csv("transcript_alignment_based_quantification.csv")
                  align_based_transcript <- align_based_transcript[, c("sample", "transcript_id", "count")]
                  align_free_transcript <- utils::read.csv("transcript_alignment_free_quantification.csv")
                  align_free_transcript <- align_free_transcript[, c("sample", "transcript_id", "count")]
                  utils::write.csv(align_based_transcript, "transcript_alignment_based_quantification.csv", row.names = FALSE)
                  utils::write.csv(align_free_transcript, "transcript_alignment_free_quantification.csv", row.names = FALSE)
                  
                  average(align_based_gene, align_free_gene, align_based_transcript, align_free_transcript)
                } else if (scRNA == TRUE) {
                  scRNA_quan(transcript, protocol, threads)
                }
            }
        }
    }
}

#' @order 2
### extract the genome and annotation files
extract_genome <- function(taxa) {
    transcript <- list.files(path = "./dehydrated", pattern = "rna\\.fna", recursive = TRUE, full.names = TRUE)
    ref_seq <- list.files(path = "./dehydrated", pattern = "fna$", recursive = TRUE, full.names = TRUE)
    ref_seq <- ref_seq[ref_seq != transcript]
    taxa <- gsub("\\s", "_", taxa)
    name1 <- paste0(taxa, ".fna")
    ref_seq1 <- paste(ref_seq, collapse = " ")
    cmd2 <- paste(ref_seq1, ">", name1)
    system2(command = "cat", args = cmd2)
    gff <- list.files(path = "./dehydrated", pattern = "gff$", recursive = TRUE, full.names = TRUE)
    name2 <- paste0(taxa, ".gff")
    cmd3 <- paste(gff, name2)
    system2(command = "mv", args = cmd3)
    name3 <- paste0("transcript_", taxa, ".fna")
    cmd4 <- paste(transcript, name3)
    system2(command = "mv", args = cmd4)
    unlink("./ncbi_dataset.zip")
    unlink("README.md")
    unlink("./dehydrated", recursive = TRUE)
}



#' Download the latest reference genome, transcript and the corresponding annotation files from NCBI.
#'
#' Download the latest reference genome and the corresponding annotation files from NCBI using the NCBI datasets command-line tool.
#' @param taxa the scientific or common name of the organism.
#' @export down_Ref
#' @order 1
#' @return None
#' @examples
#' down_Ref(taxa = 'Drosophila melanogaster')
#'
down_Ref <- function(taxa) {

    taxa_raw <- taxa
    taxa_tmp <- gsub("\\s", "_", taxa)
    genome <- paste0(taxa_tmp, ".fna")
    transcript <- paste0("transcript_", taxa_tmp, ".fna")
    annotation <- paste0(taxa_tmp, ".gff")
    
    if (!(file.exists(genome) && file.exists(transcript) && file.exists(annotation))) {
        taxa <- paste0("'", taxa, "'")
        datasets <- list.files(pattern = "^datasets$", full.names = TRUE)
        if (length(datasets) == 0) {
            print('Downloading ncbi datasets.')
            ### switch datasets according to the platform
            if (Sys.info()["sysname"] == "Linux") {
              status <- tryCatch(utils::download.file("https://ftp.ncbi.nlm.nih.gov/pub/datasets/command-line/LATEST/linux-amd64/datasets", destfile = "datasets", 
                quit = TRUE), error = function(err) {
                1
              }, warning = function(war) {
                2
              })
              datasets <- list.files(pattern = "^datasets$", full.names = TRUE)
            } else if (Sys.info()["sysname"] == "Darwin") {
              status <- tryCatch(utils::download.file("https://ftp.ncbi.nlm.nih.gov/pub/datasets/command-line/LATEST/mac/datasets", destfile = "datasets", 
                quit = TRUE), error = function(err) {
                1
              }, warning = function(war) {
                2
              })
              datasets <- list.files(pattern = "^datasets$", full.names = TRUE)
            }
            if (status){
                print("The internet connection is poor. The download of datasets is failed. Please retry later.")
                unlink(datasets)
                return(status)
            }
        }
        cmd0 <- paste("+x", datasets)
        system2(command = "chmod", args = cmd0)
        i = 0
        while (!file.exists("dehydrated/ncbi_dataset/fetch.txt")&& i <3) {
            unlink('dehydrated', recursive = TRUE)
            cmd2 <- paste("download genome taxon", taxa, "--reference --dehydrated --filename dehydrated.zip")
            print(cmd2)
            print(datasets)
            system2(command = datasets, args = cmd2, stdout = TRUE)
            if (Sys.info()["sysname"] == "Linux") {
                system2(command = 'unzip', args = 'dehydrated.zip -d dehydrated', stdout = TRUE)
            } else{
                utils::unzip("dehydrated.zip", list = FALSE, overwrite = TRUE, unzip = "internal", exdir = "dehydrated")
                }
            i = i + 1
        } 
        if(!file.exists("dehydrated/ncbi_dataset/fetch.txt")){
            status = 1
            print("The internet connection is poor and the download of reference genome and annotation files failed. Please retry later!")
            return(status)
        }
        print("Downloading the reference genome and annotation files.")
        cmd3 <- paste("rehydrate --directory ./dehydrated")
        # status <- system2(command = datasets, args = cmd3, stdout = FALSE, stderr = FALSE)
        status <- system2(command = datasets, args = cmd3)
        
        # print(status)
        if (status) {
            print("The internet connection is poor and the download of reference genome and annotation files failed. Please retry later!")
        } else {
          unlink(datasets)
          extract_genome(taxa_raw)
        }
        unlink("dehydrated.zip")
        
    } else {
        status = 0
        print("The reference genome and annotation files already exist")
    }
    return(status)
}

#' @order 2
### produce fastqc reports
fastqc_r <- function(threads, fq.dir, scRNA, fastqc_add) {
    status <- NULL
    if (scRNA == FALSE) {
        files_fastq <- list.files(fq.dir, pattern = ".fastq$", recursive = FALSE, full.names = TRUE)
        if (length(files_fastq)) {
            for (f in files_fastq) {
                cmd = paste(f, " --threads ", threads, fastqc_add)
                system2(command = "fastqc", args = cmd)  # invoke command
            }
            status <- 0
        } else {
            print("No fastq files are found in the work directory.")
            status <- 1
        }
    } else if (scRNA == TRUE) {
        read <- list.files(pattern = ".*1\\.fastq$", full.names = FALSE)
        read_seq <- c()
        barcode_seq <- c()
        if (length(read)) {
            for (f in read) {
                name <- gsub("_1.fastq", "", f)
                read1 <- paste0(name, "_1.fastq")
                read2 <- paste0(name, "_2.fastq")
                read_seq <- c(read_seq, read2)
                barcode_seq <- c(barcode_seq, read1)
                cmd1 <- paste0("-n 1 ", read1, " | grep -o length=[0-9]* | cut -d '=' -f 2")
                leg_1 <- as.numeric(system2(command = "head", args = cmd1))
                cmd2 <- paste0("-n 1 ", read2, " | grep -o length=[0-9]* | cut -d '=' -f 2")
                leg_2 <- as.numeric(system2(command = "head", args = cmd2))
                if (leg_1 > leg_2) {
                  read_seq <- c(read_seq, read1)
                  barcode_seq <- c(barcode_seq, read2)
                }
            }
            for (d in read_seq) {
                cmd = paste(d, "--threads", threads, fastqc_add)
                system2(command = "fastqc", args = cmd)  # invoke command
            }
            status <- 0
        } else {
            print("No fastq files are found in the work directory.")
            status <- 1
        }
    }
}

utils::globalVariables(c("module", "status"))


#' Quality assessments of raw sequence data.
#'
#' Per base sequence quality, per sequence quality scores and adapter content of raw sequence data are assessed and tested based on FastQC.
#' @param threads the number of threads to be used. Default is 4.
#' @param scRNA logic, whether single-cell RNA-seq is quantified or not. Default is FALSE.
#' @param fastqc_add additional parameters to customize FASTQC for quality control. Default is NULL.
#' @export qc_test
#' @order 1
#' @return Problematic samples' names
#' @examples
#'
#' qc_test(scRNA = TRUE)
#'
#'

qc_test <- function(threads = 4, scRNA = FALSE, fastqc_add = NULL) {
    existence <- check_dep(dependancy = "fastqc")
    if (existence == TRUE) {
        fq.dir = getwd()
        qc.dir = getwd()
        status <- fastqc_r(threads, fq.dir, scRNA, fastqc_add)
        if (status == 0) {
            qc <- fastqcr::qc_aggregate(qc.dir, progressbar = FALSE)
            if (length(qc)) {
                index_ad <- subset(qc, module == "Adapter Content" & status == "FAIL", select = sample)
                index_sq <- subset(qc, module == "Per base sequence quality" & status == "FAIL", select = sample)
                index_sqs <- subset(qc, module == "Per sequence quality scores" & status == "FAIL", select = sample)
                index <- list(adapter = index_ad, base_quality = index_sq, sequence_score = index_sqs)
                return(index)
            }
        }
    } else print("FASTQC is not found. Please install it.")
}

test_that("ncbi datasets url works", {
    status <- 0
    if (Sys.info()["sysname"] == "Linux") {
        status <- tryCatch(utils::download.file("https://ftp.ncbi.nlm.nih.gov/pub/datasets/command-line/LATEST/linux-amd64/datasets", destfile = "datasets", 
                                                quit = TRUE), error = function(err) {
                                                    1
                                                }, warning = function(war) {
                                                    2
                                                })
        datasets <- list.files(pattern = "^datasets$", full.names = TRUE)
    } else if (Sys.info()["sysname"] == "Darwin") {
        status <- tryCatch(utils::download.file("https://ftp.ncbi.nlm.nih.gov/pub/datasets/command-line/LATEST/mac/datasets", destfile = "datasets", 
                                                quit = TRUE), error = function(err) {
                                                    1
                                                }, warning = function(war) {
                                                    2
                                                })
        datasets <- list.files(pattern = "^datasets$", full.names = TRUE)
    } else if (Sys.info()["sysname"] == "Windows") print("Please use Windows Subsystem for Linux.")
    expect_equal(status, 0)
})

---
title: "BP4RNAseq vignette"
author: Shanwen Sun, Lei Xu, Quan Zou
date: 2020.07.25
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{BP4RNAseq vignette}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
references:
- id: Srivastava2019
  title: Alevin efficiently estimates accurate gene abundances from dscRNA-seq data
  author: 
  - family: Srivastava
    given: A.
  - family: Malik
    given: L.
  - family: Smith
    given: T.    
  - family: Sudbery
    given: I.
  - family: Patro
    given: R.
  volume: 20
  DOI: 10.1186/s13059-019-1670-y
  publisher: Genome Biology
  page: 16
  type: article-journal
  issued:
    year: 2019
    
- id: Lachmann2020
  title: Interoperable RNA-Seq analysis in the cloud
  author: 
  - family: Lachmann
    given: A.
  - family: Clarke
    given: D. J. B.
  - family: Torre
    given: D.    
  - family: Xie
    given: Z. R.
  - family: Ma'ayan
    given: A.
  volume: 1863
  DOI: 10.1016/j.bbagrm.2020.194521
  publisher: Biochimica Et Biophysica Acta-Gene Regulatory Mechanisms
  issue: 6
  page: 1-11
  type: article-journal
  issued:
    year: 2020
---

```{r, include = FALSE}
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>"
)
```

# Introduction

The assessment of gene expression is central to uncovering the functions of the genome, understanding the regulation of development and investigating the molecular mechanisms that underlie cancer and other diseases. RNA-sequencing (RNA-seq) now is the routine to assess the genome wide gene expression due to its high speed, accuracy and reproducibility, and low cost. An enormous volume of RNA-seq data have been accumulating and deposited in public data repositories, such as the Gene Expression Omnibus (GEO) and the Sequence Read Archive (SRA). Retrospectively analyzing these data or conducting a brand new RNA-seq study is fundamentally important for researchers. However, processing raw reads of RNA-seq data, no matter public or newly sequenced data, involves a lot of specialized tools and technical configurations that are often unfamiliar and time-consuming to learn for non-bioinformatics researchers. For example, when working with public RNA-seq data, researchers need to download the RNA-seq data, convert data to FASTQ format, check the sequencing type (i.e., single-end or pair-end), do the quality control (when needed, trim adapters and poor quality reads), download the reference genome, transcript and annotation file, align reads to the reference genome or transcript and quantify gene expression, etc. These steps and the details that they involve are even tedious for bioinformatic scientist. The goal of BP4RNAseq is to make the RNA-seq analysis smooth and easy and to minimize efforts from researchers. The package offers several benefits to researchers. First, the package is a highly automated tool. It can take only two nontechnical parameters and output six formatted gene expression quantification at gene and transcript levels. Second, it improves the accuracy and sensitivity of RNA-seq analyses by using an optimized pipeline. Third, it offers individual tools to provide users full control to fine tune precisely how individual steps are optimized. This can allow users to inspect intermediate outputs and thus to further improve the accuracy and sensitivity of RNA-seq analyses. Users can also use the package as a toolbox to run the exact tools that suit their needs. Last but not least, the package applies to both retrospective and newly generated bulk RNA-seq data analyses and is also applicable for single-cell RNA-seq analyses based on the Alevin algorithm ^[Srivastava, A., et al. Alevin efficiently estimates accurate gene abundances from dscRNA-seq data. Genome Biol 2019;20:16.].



# Operating System Requirements

BP4RNAseq runs in Windows (Subsystem for Linux), Linux and macOS.

# Dependencies 

The BP4RNAseq requires the following utilities:

* [SRA Toolkit >= 2.10.3](https://trace.ncbi.nlm.nih.gov/Traces/sra/sra.cgi?view=toolkit_doc)
* [Entrez Direct >= 13.3](https://www.ncbi.nlm.nih.gov/books/NBK179288/)
* [FastQC >= v0.11.9](https://www.bioinformatics.babraham.ac.uk/projects/fastqc/)
* [Cutadapt >= 2.10](https://cutadapt.readthedocs.io/en/stable/)
* [SAMtools >= 1.9](http://www.htslib.org/)
* [HISAT2 >= 2.2.0](http://daehwankimlab.github.io/hisat2/)
* [StringTie >= 2.1.1](https://ccb.jhu.edu/software/stringtie/)
* [Salmon >= 1.2.1](https://combine-lab.github.io/salmon/)  
* [jq >= 1.6](https://stedolan.github.io/jq/)
* [R >= 4.0.0](https://www.r-project.org/)

Users can install these dependencies manually.  

Alternatively, we provide a bash script to aid users to install all the dependencies based on [conda](https://docs.conda.io/en/latest/). The script uses Wget, which is pre-installed on most Linux distributions such as Windows Subsystem for Linux, to download conda. If wget is not installed, users can easily install it with the following commands. 

* Installing Wget on Ubuntu and Debian or Windows Subsystem equivalents
```r
sudo apt-get update 
sudo apt-get install -y wget
```
* Installing Wget on CentOS and Fedora or Windows Subsystem equivalents
```r
sudo yum install wget
```
* Installing Wget on macOS with Homebrew
```r
brew install wget
```

With Wget installed, users can install all the dependencies with the following commands:
``` r
wget https://raw.githubusercontent.com/sunshanwen/BP4RNAseq/master/install_depends.sh
chmod +x install_depends.sh
./install_depends.sh
./use_conda.sh
```


# Installation

<!-- You can install the released version of BP4RNAseq from [CRAN](https://CRAN.R-project.org) with: -->

<!-- ``` r -->
<!-- #install.packages("BP4RNAseq") # remove comments later -->
<!-- ``` -->

<!-- And the development version from [GitHub](https://github.com/) with: -->

<!-- ``` r -->
<!-- # install.packages("devtools") -->
<!-- devtools::install_github("sunshanwen/BP4RNAseq") -->
<!-- ``` -->

You can install BP4RNAseq from [GitHub](https://github.com/) with:

``` r
devtools::install_github("sunshanwen/BP4RNAseq")
```


# Usage

## Bulk RNA-seq analyses
The functions in BP4RNAseq are integrated into two main functions: down2quan for public RNA-seq data, fastq2quan for newly generated RNA-seq data. 

down2quan requires no input data and can receive only two nontechnical parameters. The parameter `accession` specifies the accession id of the target public RNA-seq data in the Gene Expression Omnibus (GEO) or the Sequence Read Archive (SRA). The accession id can be of a whole 'BioProject' or multiple 'BioSample'. The parameter `taxa` offers the scientific or common name of the organism investigated. A simple example 
``` r
library(BP4RNAseq)
down2quan(accession=c("SRR11486115","SRR11486114"), taxa="Drosophila melanogaster")
```
will download the public RNA-seq data of two 'BioSample' with accession id "SRR11486115" and "SRR11486114", respectively, and the latest reference genome, transcript and annotation data of Drosophila melanogaster, do the quality control (filter out the poor-quality reads and contaminations), reads alignments and gene expression quantification based on both alignment-free and alignment-based workflows in the work directory. During the quality control procedure, if the contamination of the adapter exists, the program will automatically detect the adapter sequence to trim. However, an option is given to the users to provide the adapter sequence if they know it. 

fastq2quan works with local RNA-seq data in fastq formats. It needs two nontechnical parameters at a minimum, i.e., `taxa` as explained above and `pair`which specifies the sequencing type with `single` for single-end (SE) reads or `paired` for paired-end (PE) reads. Users should place all the fastq files in the work directory. A simple example 
``` r
library(BP4RNAseq)
fastq2quan(taxa="Drosophila melanogaster", pair = "single")
```
will download the latest reference genome, transcript and annotation data of Drosophila melanogaster, and do the quality control, reads alignments and gene expression quantification using the local RNA-seq data based on both alignment-free and alignment-based workflows as the program down2quan do. 

Both programs support the parallel computing, which is specified by the `threads` parameter. 

Outputs from both functions are two gene count matrixes and two transcript count matrixes based on the alignment-based workflow and the alignment-free workflow, and corresponding average matrixes over two workflows. These outputs can be directly processed with [DESeq2](https://bioconductor.org/packages/release/bioc/html/DESeq2.html), [edgeR](https://bioconductor.org/packages/release/bioc/html/edgeR.html) or [limma](https://bioconductor.org/packages/release/bioc/html/limma.html). Researchers may use the averages for downstream analyses ^[Lachmann, A., et al. Interoperable RNA-Seq analysis in the cloud. Biochim. Biophys. Acta-Gene Regul. Mech. 2020;1863(6):1-11.]. Alternatively, we recommend to decide the type of data to use based on their consistencies with qPCR results if available or/and the results from the downstream analyses.


## Single-cell RNA-seq analyses

down2quan and fastq2quan can also be extended to process single-cell RNA-seq data by setting the `scRNA` parameter to be 'TRUE' and specifying the protocols. Currently, dropseq, chromium and chromiumV3 are supported protocols. A simple example 
``` r
library(BP4RNAseq)
down2quan(accession=c("SRR11402955","SRR11402974"), taxa="Homo sapiens", scRNA = TRUE, protocol = "dropseq")
```
will download the public single-cell RNA-seq data from two 'BioSample' with accession id "SRR11402955" and "SRR11402974", respectively, and the latest reference genome, transcript and annotation data of Homo sapiens, do the quality control, reads alignments and gene expression quantification based on the Alevin workflow.

Alternatively,
``` r
library(BP4RNAseq)
fastq2quan(taxa="Homo sapiens", scRNA = TRUE, protocol = "dropseq")
```
can preprocess local single-cell RNA-seq data in fastq formats. The data are paired-end reads with one read containing cellular barcode and unique molecule identifier (UMI) and the other read being the RNA sequence.

The outputs of down2quan and fastq2quan are gene count matrix compressed in binary format, and gene ids, barcode + UMI and tier categorization in three separate files. These outputs can be further processed with [tximport](https://bioconductor.org/packages/devel/bioc/vignettes/tximport/inst/doc/tximport.html) and [Seurat](https://satijalab.org/seurat/).


## Using BP4RNAseq as a toolbox for RNA-seq analyses and customizing gene expression quantification to improve the sensitivity and accuracy of RNA-seq analyses.

BP4RNAseq offers individual tools to users. These tools can allow users to run the exact tools that suit their needs. Specifically, these tools are:
 
* sra_download(), which can download RNA-seq data from public repositories. 
* sra2fastq(), which converts SRA files into FASTQ format.
* qc_test(), which assesses the quality of reads.
* adapter_trim(), which trims adapter.
* quality_trim(), which trims poor quality reads.
* down_Ref(), which downloads the latest reference genome, transcript and annotation files.
* align_based_quan(), which performs the alignment-based gene expression quantification, including reads alignment and assembly.
* align_free_quan(), which performs the alignment-free gene expression quantification.
* scRNA_quan(), which performs the single-cell RNA-seq gene expression quantification.


Additionally, these individual tools provide users full control to fine tune precisely how individual steps are optimized. This can allow experienced users to further improve the accuracy and sensitivity of RNA-seq analyses. For example, setting the optional parameter ` salmon_quan_add` of align_free_quan() as `salmon_quan_add = "--useEM --gcBias"` will allow users to apply the standard EM algorithm to optimize abundance estimates and in the mean time to correct for fragment-level GC biases in the input data when performing the alignment-free workflow. Details about the optional customizing setting in each tool can be found in package help page. 

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/average.R
\name{average}
\alias{average}
\title{Take averages with alignment-free and alignment-based quantification}
\usage{
average(based_gene, free_gene, based_transcript, free_transcript)
}
\arguments{
\item{based_gene}{gene count matrix assessed based on alignment-based workflow}

\item{free_gene}{gene count matrix assessed based on alignment-free workflow}

\item{based_transcript}{transcript count matrix assessed based on alignment-based workflow}

\item{free_transcript}{transcript count matrix assessed based on alignment-free workflow}
}
\value{
the average of gene expression quantification with alignment-free and alignment-based quantification workflow
}
\description{
Take averages with alignment-free and alignment-based quantification
}
\examples{

align_based_gene <- system.file(
'extdata', 'test_gene_alignment_based_quantification.csv', 
package = 'BP4RNAseq'
)
align_free_gene <- system.file(
'extdata', 'test_gene_alignment_free_quantification.csv', 
package = 'BP4RNAseq'
)
align_based_transcript <- system.file(
'extdata', 'test_transcript_alignment_based_quantification.csv',
package = 'BP4RNAseq'
)
align_free_transcript <- system.file(
'extdata', 'test_transcript_alignment_free_quantification.csv', 
package = 'BP4RNAseq'
)
align_based_gene <- read.csv(align_based_gene, header = TRUE)
align_free_gene <- read.csv(align_free_gene, header = TRUE)
align_based_transcript <- read.csv(align_based_transcript, header = TRUE)
align_free_transcript <- read.csv(align_free_transcript, header = TRUE)
average(
align_based_gene, align_free_gene, 
align_based_transcript, align_free_transcript
)
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/align_based_quan.R
\name{align_based_quan}
\alias{align_based_quan}
\title{alignment-based workflow}
\usage{
align_based_quan(
  pair,
  taxa,
  genome,
  annotation,
  novel_transcript = FALSE,
  threads = 4,
  hisat2_build_add = NULL,
  hisat2_add = NULL,
  stringtie_add = NULL
)
}
\arguments{
\item{pair}{'single' for single-end (SE) or 'paired' for paired-end (PE) reads.}

\item{taxa}{the scientific or common name of the organism.}

\item{genome}{the reference genome.}

\item{annotation}{the annotation file.}

\item{novel_transcript}{logic, whether identifying novel transcripts is expected or not. Default is FALSE.}

\item{threads}{the number of threads to be used. Default is 4.}

\item{hisat2_build_add}{additional parameters to customize hisat2 build command. Default is NULL.}

\item{hisat2_add}{additional parameters to customize hisat2 command. Default is NULL.}

\item{stringtie_add}{additional parameters to customize stringtie command. Default is NULL.}
}
\value{
None
}
\description{
alignment-based workflow
}
\examples{
\dontrun{
test_Homo_sapiens.fna <- system.file('extdata', 'test_Homo_sapiens.fna', package = 'BP4RNAseq')
test_Homo_sapiens.gff <- system.file('extdata', 'test_Homo_sapiens.gff', package = 'BP4RNAseq')
align_based_quan(pair = 'paired', taxa = 'Homo sapiens', 
genome = test_Homo_sapiens.fna, 
annotation = test_Homo_sapiens.gff, novel_transcript = FALSE)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/check_dep.R
\name{check_dep}
\alias{check_dep}
\title{Check the existence of dependancies.}
\usage{
check_dep(dependancy)
}
\arguments{
\item{dependancy}{the dependancy to check.}
}
\value{
TRUE if the dependancy exists, otherwise FALSE.
}
\description{
Trim samples with adapter using cutadapt based on the results from quality_test().
}
\examples{

check_dep(dependancy = 'cutadapt')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/align_free_quan.R
\name{align_free_quan}
\alias{align_free_quan}
\title{Alignment-free expression quantification with Salmon at gene and transcript levels.}
\usage{
align_free_quan(
  pair,
  genome,
  transcript,
  annotation,
  threads = 4,
  salmon_index_add = NULL,
  salmon_quan_add = NULL
)
}
\arguments{
\item{pair}{'single' for single-end (SE) or 'paired' for paired-end (PE) reads.}

\item{genome}{the reference genome.}

\item{transcript}{the reference transcript}

\item{annotation}{the annotation file.}

\item{threads}{the number of threads to be used. Default is 4.}

\item{salmon_index_add}{additional parameters to customize salmon index command. Default is NULL.}

\item{salmon_quan_add}{additional parameters to customize salmon quan command. Default is NULL.}
}
\value{
None
}
\description{
Alignment-free expression quantification with Salmon at gene and transcript levels.
}
\examples{

align_free_quan(
pair = 'paired', 'test_Homo_sapiens.fna', 
'test_transcript_Homo_sapiens.fna','test_Homo_sapiens.gff'
)



}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/scRNA_quan.R
\name{scRNA_quan}
\alias{scRNA_quan}
\title{scRNA expression quantification with Salmon.}
\usage{
scRNA_quan(
  transcript,
  protocol,
  threads = 4,
  salmon_index_add = NULL,
  salmon_alevin_add = NULL
)
}
\arguments{
\item{transcript}{the reference transcript}

\item{protocol}{the single-cell RNA sequencing protocol: dropseq, chromium, or chromiumV3.}

\item{threads}{the number of threads to be used. Default is 4.}

\item{salmon_index_add}{additional parameters to customize salmon index command. Default is NULL.}

\item{salmon_alevin_add}{additional parameters to customize salmon alevin command. Default is NULL.}
}
\value{
None
}
\description{
scRNA expression quantification with Salmon.
}
\examples{

scRNA_quan(transcript = 'test_transcript_Homo_sapiens.fna', protocol = 'dropseq')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/down2quan.R
\name{down2quan}
\alias{down2quan}
\title{Download the RNA-seq samples from the NCBI and produce the quantifications of gene expression.}
\usage{
down2quan(
  accession,
  dir = getwd(),
  taxa,
  novel_transcript = FALSE,
  threads = 4,
  scRNA = FALSE,
  protocol
)
}
\arguments{
\item{accession}{the bioproject accession code for the RNA-seq samples deposited in the NCBI.}

\item{dir}{the working directory. Default is the current working directory.}

\item{taxa}{the scientific or common name of the organism.}

\item{novel_transcript}{logic, whether identifying novel transcripts is expected or not. Default is FALSE.}

\item{threads}{the number of threads to be used. Default is 4.}

\item{scRNA}{logic, whether single-cell RNA-seq is quantified or not. Default is FALSE.}

\item{protocol}{the single-cell RNA sequencing protocol: dropseq, chromium, or chromiumV3.}
}
\value{
None
}
\description{
Download the RNA-seq samples from the NCBI and produce the quantifications of gene expression with alignment-based and alignment-free workflows.
}
\examples{

down2quan(
accession = 'SRR11427582', dir = getwd(), 
taxa = 'Drosophila melanogaster', novel_transcript = FALSE, scRNA = FALSE
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/qc_test.R
\name{qc_test}
\alias{qc_test}
\title{Quality assessments of raw sequence data.}
\usage{
qc_test(threads = 4, scRNA = FALSE, fastqc_add = NULL)
}
\arguments{
\item{threads}{the number of threads to be used. Default is 4.}

\item{scRNA}{logic, whether single-cell RNA-seq is quantified or not. Default is FALSE.}

\item{fastqc_add}{additional parameters to customize FASTQC for quality control. Default is NULL.}
}
\value{
Problematic samples' names
}
\description{
Per base sequence quality, per sequence quality scores and adapter content of raw sequence data are assessed and tested based on FastQC.
}
\examples{

qc_test(scRNA = TRUE)


}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/quality_trim.R
\name{quality_trim}
\alias{quality_trim}
\title{Trim poor quality samples using cutadapt.}
\usage{
quality_trim(
  per_base,
  per_seq,
  pair,
  threads = 4,
  scRNA = FALSE,
  cutadapt_add = NULL
)
}
\arguments{
\item{per_base}{samples with poor per base sequence quality.}

\item{per_seq}{samples with poor per sequence quality scores.}

\item{pair}{'single' for single-end (SE) or 'paired' for paired-end (PE) reads.}

\item{threads}{the number of threads to be used. Default is 4.}

\item{scRNA}{logic, whether single-cell RNA-seq is quantified or not. Default is FALSE.}

\item{cutadapt_add}{additional parameters to customize cutadapt to trim poor quality reads. Default is NULL}
}
\value{
None
}
\description{
Trim poor quality samples using cutadapt based on the results from quality_test().
}
\examples{

quality_trim(per_base = 'test_read_2.fastq', per_seq = 'test_read_2.fastq',
             pair = 'paired', threads = 4, scRNA = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sra2fastq.R
\name{sra2fastq}
\alias{sra2fastq}
\title{Convert SRA format to fastq format and return the sequencing type, i.e., single-end (SE) or paired-end (PE) reads.}
\usage{
sra2fastq(threads = 4)
}
\arguments{
\item{threads}{the number of threads to be used. Default is 4.}
}
\value{
A string indicates single-end (SE) or paired-end (PE) reads.
}
\description{
Convert all SRA files to fastq files and return the sequencing type, i.e., single-end (SE) or paired-end (PE) reads.
}
\examples{

sra_download(accession = 'SRR11427582')
sra2fastq()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fastq2quan.R
\name{fastq2quan}
\alias{fastq2quan}
\title{Produce the quantifications of gene expression based on the fastq files.}
\usage{
fastq2quan(
  threads = 4,
  dir = getwd(),
  pair,
  taxa,
  novel_transcript = FALSE,
  scRNA = FALSE,
  protocol = NULL
)
}
\arguments{
\item{threads}{the number of threads to be used. Default is 4.}

\item{dir}{the working directory. Default is the current working directory.}

\item{pair}{'single' for single-end (SE) reads or 'paired' for paired-end (PE) reads.}

\item{taxa}{the scientific or common name of the organism.}

\item{novel_transcript}{logic, whether identifying novel transcripts is expected or not. Default is FALSE.}

\item{scRNA}{logic, whether single-cell RNA-seq is quantified or not. Default is FALSE.}

\item{protocol}{the single-cell RNA sequencing protocol: dropseq, chromium, or chromiumV3.}
}
\value{
None
}
\description{
Produce the quantifications of gene expression based on the fastq files with alignment-based and alignment-free workflows.
}
\examples{

sra_download(accession = 'SRR11427582')
sra2fastq()
fastq2quan(pair = 'single', taxa = 'Drosophila melanogaster')
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/down_Ref.R
\name{down_Ref}
\alias{down_Ref}
\title{Download the latest reference genome, transcript and the corresponding annotation files from NCBI.}
\usage{
down_Ref(taxa)
}
\arguments{
\item{taxa}{the scientific or common name of the organism.}
}
\value{
None
}
\description{
Download the latest reference genome and the corresponding annotation files from NCBI using the NCBI datasets command-line tool.
}
\examples{
down_Ref(taxa = 'Drosophila melanogaster')

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/adapter_trim.R
\name{adapter_trim}
\alias{adapter_trim}
\title{Trim samples with adapter using cutadapt.}
\usage{
adapter_trim(sample, pair, threads = 4, scRNA = FALSE, cutadapt_add = NULL)
}
\arguments{
\item{sample}{samples with adapter.}

\item{pair}{'single' for single-end (SE) or 'paired' for paired-end (PE) reads.}

\item{threads}{the number of threads to be used. Default is 4.}

\item{scRNA}{logic, whether single-cell RNA-seq is quantified or not. Default is FALSE.}

\item{cutadapt_add}{additional parameters to customize cutadapt to trim adapter. Default is NULL.}
}
\value{
None
}
\description{
Trim samples with adapter using cutadapt based on the results from quality_test().
}
\examples{

adapter_trim(sample = 'test_read_2.fastq', pair = 'paired', scRNA = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sra_download.R
\name{sra_download}
\alias{sra_download}
\title{Download RNA-seq samples from the NCBI.}
\usage{
sra_download(accession, dir = getwd())
}
\arguments{
\item{accession}{the bioproject accession code for the RNA-seq samples deposited in the NCBI.}

\item{dir}{the working directory. Default is the current working directory.}
}
\value{
the status of the download (0 for success, 1 for error).
}
\description{
Download RNA-seq samples from the NCBI using SRA Toolkit and Entrez Direct based on the accession code.
}
\examples{
sra_download(accession = 'SRR11427582')

}
